# Autogenerated by Astropy-affiliated package astropy_helpers's setup.py on 2015-11-17 10:45:33.810004
from __future__ import unicode_literals
import datetime

version = "1.0.5"
githash = "160b48587680e11f12b66a51641a2601b869d1fa"


major = 1
minor = 0
bugfix = 5

release = True
timestamp = datetime.datetime(2015, 11, 17, 10, 45, 33, 810004)
debug = False

try:
    from ._compiler import compiler
except ImportError:
    compiler = "unknown"

try:
    from .cython_version import cython_version
except ImportError:
    cython_version = "unknown"
